@@ -82,6 +82,10 @@ setClass("triangularMatrix", contains = c("Matrix", "VIRTUAL"),
 	 prototype = prototype(uplo = "U", diag = "N"),
 	 validity = function(object) .Call(triangularMatrix_validate, object))
 
+setClass("packedMatrix", contains = c("Matrix", "VIRTUAL"),
+	 slots = c(uplo = "character"),
+	 prototype = prototype(uplo = "U"),
+	 validity = function(object) .Call(packedMatrix_validate, object))
 
 ## Virtual class of numeric matrices
 setClass("dMatrix", contains = c("Matrix", "VIRTUAL"), slots = c(x = "numeric"),
@@ -154,6 +158,20 @@ setClass("CsparseMatrix", contains = c("sparseMatrix", "VIRTUAL"),
          validity = function(object) .Call(Csparse_validate, object)
          )
 
+if(FALSE) { ## in theory.. would be neat for  new("dgCMatrix", Dim = c(3L,3L))
+setMethod("initialize", "CsparseMatrix", function(.Object, ...) {
+    .Object <- callNextMethod()
+    .Object@p <- integer(.Object@Dim[2L] + 1L)
+    .Object
+})
+
+setMethod("initialize", "RsparseMatrix", function(.Object, ...) {
+    .Object <- callNextMethod()
+    .Object@p <- integer(.Object@Dim[1L] + 1L)
+    .Object
+})
+}# not yet (fails)
+
 setClass("RsparseMatrix", contains = c("sparseMatrix", "VIRTUAL"),
 	 slots = c(p = "integer", j = "integer"),
 	 prototype = prototype(p = 0L),# to be valid
@@ -203,7 +221,7 @@ setClass("dtrMatrix",
 
 ## numeric, dense, packed, triangular matrices
 setClass("dtpMatrix",
-	 contains = c("ddenseMatrix", "triangularMatrix"),
+	 contains = c("ddenseMatrix", "triangularMatrix", "packedMatrix"),
 	 validity = function(object) .Call(dtpMatrix_validate, object))
 
 
@@ -214,7 +232,7 @@ setClass("dsyMatrix",
 
 ## numeric, dense, packed symmetric matrices
 setClass("dspMatrix",
-	 contains = c("ddenseMatrix", "symmetricMatrix"),
+	 contains = c("ddenseMatrix", "symmetricMatrix", "packedMatrix"),
 	 validity = function(object) .Call(dspMatrix_validate, object))
 
 ## numeric, dense, non-packed, positive-definite, symmetric matrices
@@ -243,7 +261,7 @@ setClass("ltrMatrix",
 
 ## logical, dense, packed, triangular matrices
 setClass("ltpMatrix",
-	 contains = c("ldenseMatrix", "triangularMatrix"))
+	 contains = c("ldenseMatrix", "triangularMatrix", "packedMatrix"))
 
 ## logical, dense, non-packed symmetric matrices
 setClass("lsyMatrix",
@@ -252,7 +270,7 @@ setClass("lsyMatrix",
 
 ## logical, dense, packed symmetric matrices
 setClass("lspMatrix",
-	 contains = c("ldenseMatrix", "symmetricMatrix"),
+	 contains = c("ldenseMatrix", "symmetricMatrix", "packedMatrix"),
 	 validity = function(object) .Call(dspMatrix_validate, object)
 	 ## "dsp", "lsp" and "nsp" have the same validate
 	 )
@@ -271,7 +289,7 @@ setClass("ntrMatrix",
 
 ## logical, dense, packed, triangular matrices
 setClass("ntpMatrix",
-	 contains = c("ndenseMatrix", "triangularMatrix"))
+	 contains = c("ndenseMatrix", "triangularMatrix", "packedMatrix"))
 
 ## logical, dense, non-packed symmetric matrices
 setClass("nsyMatrix",
@@ -280,7 +298,7 @@ setClass("nsyMatrix",
 
 ## logical, dense, packed symmetric matrices
 setClass("nspMatrix",
-	 contains = c("ndenseMatrix", "symmetricMatrix"),
+	 contains = c("ndenseMatrix", "symmetricMatrix", "packedMatrix"),
 	 validity = function(object) .Call(dspMatrix_validate, object)
 	 ## "dsp", "lsp" and "nsp" have the same validate
 	 )
@@ -288,7 +306,7 @@ setClass("nspMatrix",
 
 ## 'diagonalMatrix' already has validity checking
 ## diagonal, numeric matrices; "dMatrix" has 'x' slot :
-setClass("ddiMatrix", contains = c("diagonalMatrix", "dMatrix"))# or "dMatrix"
+setClass("ddiMatrix", contains = c("diagonalMatrix", "dMatrix"))
 ## diagonal, logical matrices; "lMatrix" has 'x' slot :
 setClass("ldiMatrix", contains = c("diagonalMatrix", "lMatrix"))
 
@@ -737,7 +755,7 @@ setClassUnion("geMatrix", members = c("dgeMatrix", "lgeMatrix", "ngeMatrix"))
 
 ## --- "General" (not Matrix at all) ----
 
-## e.g. for "Arith" methods:
+## e.g. for "Arith" methods, NB: --> see "numericVector" below (incl "integer")
 setClassUnion("numLike", members = c("numeric", "logical"))
 
 ##setClassUnion("numIndex", members = "numeric")
@@ -814,19 +832,10 @@ setClassUnion("atomicVector", ## "double" is not needed, and not liked by some
 	      members = c("logical", "integer", "numeric",
 			  "complex", "raw", "character"))
 
+## NB: --> see "numLike" above
+if(FALSE) # not used anywhere
 setClassUnion("numericVector", members = c("logical", "integer", "numeric"))
 
-setClassUnion("Mnumeric", members = c("numericVector", "Matrix"))
-## not "matrix" as that extends "vector" and contains "character", "structure" ...
-
-setValidity("Mnumeric",
-            function(object) {
-                if(is.numeric(object) ||
-                   is.logical(object) ||
-                   inherits(object, "Matrix")) return(TRUE)
-                ## else
-                "Not a valid 'Mnumeric' class object"
-		})
 
 
 
