@@ -7,8 +7,13 @@
 ###               -----    -------        -----
 ### where 'Arith'   :=  '"+"', '"-"', '"*"', '"^"', '"%%"', '"%/%"', '"/"'
 ###       'Compare' := '"=="', '">"', '"<"', '"!="', '"<="', '">="'
-###       'Logic'   :=  '"&"', '"|"'  (( but *not* '"!"' since that has
-###			                 only one argument ))
+###       'Logic'   :=  '"&"', '"|"'
+###		but *not* '"!"' since that has only one argument : >>>>> ./not.R
+
+## cache them [rather in package 'methods' ??]
+.ArithGenerics   <- getGroupMembers("Arith")
+.CompareGenerics <- getGroupMembers("Compare")
+.LogicGenerics   <- getGroupMembers("Logic")
 
 ## find them with M-x grep -E 'Method\("(Ops|Compare|Arith|Logic)"' *.R
 ##                --------
@@ -22,7 +27,7 @@
 
 ##-------- originally from ./Matrix.R --------------------
 
-## Some ``Univariate'' "Arith":
+## Some ``Univariate'' "Arith" (univariate := 2nd argument 'e2' is missing)
 setMethod("+", signature(e1 = "Matrix", e2 = "missing"), function(e1,e2) e1)
 
 ## "fallback":
@@ -32,7 +37,7 @@ setMethod("-", signature(e1 = "Matrix", e2 = "missing"),
 	      0-e1
 	  })
 setMethod("-", signature(e1 = "denseMatrix", e2 = "missing"),
-	  function(e1, e2) { e1@x <- -e1@x; e1 })
+	  function(e1, e2) { e1@x <- -e1@x; .empty.factors(e1); e1 })
 
 ## "diagonalMatrix" -- only two cases -- easiest to do both
 setMethod("-", signature(e1 = "ddiMatrix", e2 = "missing"),
@@ -43,6 +48,7 @@ setMethod("-", signature(e1 = "ddiMatrix", e2 = "missing"),
                   }
 		  else ## diag == "N" -> using 'x' slot
                       e1@x <- -e1@x
+              .empty.factors(e1)
 	      e1
 	  })
 setMethod("-", signature(e1 = "ldiMatrix", e2 = "missing"),
@@ -71,12 +77,28 @@ setMethod("Ops", signature(e1 = "NULL", e2 = "Matrix"),
 
 ## bail-outs -- on highest possible level, hence "Ops", not "Compare"/"Arith" :
 .bail.out.Ops <- function(e1, e2) {
-    if(is(e1, "Matrix") && is(e2, "Matrix"))
-        dimCheck(e1,e2)
+    if(is(e1, "mMatrix") && is(e2, "mMatrix"))
+	dimCheck(e1,e2)
     .bail.out.2(.Generic, class(e1), class(e2))
-          }
-setMethod("Ops", signature(e1 = "Matrix", e2 = "ANY"), .bail.out.Ops)
-setMethod("Ops", signature(e1 = "ANY", e2 = "Matrix"), .bail.out.Ops)
+}
+setMethod("Ops", signature(e1 = "Matrix", e2 = "ANY"),
+	  function(e1, e2) {
+	      if(is(e1, "mMatrix") && is(e2, "mMatrix")) dimCheck(e1,e2)
+	      if(is.matrix(e2) && identical(e2, as.matrix(e2)) &&
+		 is.object(e2) && !isS4(e2)) # e.g. for "table"
+		  callGeneric(e1, unclass(e2))
+	      else
+		  .bail.out.2(.Generic, class(e1), class(e2))
+	  })
+setMethod("Ops", signature(e1 = "ANY", e2 = "Matrix"),
+	  function(e1, e2) {
+	      if(is(e1, "mMatrix") && is(e2, "mMatrix")) dimCheck(e1,e2)
+	      if(is.matrix(e1) && identical(e1, as.matrix(e1)) &&
+		 is.object(e1) && !isS4(e1)) # e.g. for "table"
+		  callGeneric(unclass(e1), e2)
+	      else
+		  .bail.out.2(.Generic, class(e1), class(e2))
+	  })
 rm(.bail.out.Ops)
 
 ## "General principle"
@@ -108,6 +130,7 @@ setMethod("Compare", signature(e1 = "Matrix", e2 = "Matrix"),
 .Ops.via.x <- function(e1,e2) {
     dimCheck(e1, e2)
     e1@x <- callGeneric(e1@x, e2@x)
+    .empty.factors(e1)
     e1
 }
 
@@ -131,10 +154,9 @@ Cmp.Mat.atomic <- function(e1, e2) { ## result will inherit from "lMatrix"
     cl <- class(e1)
     if((l2 <- length(e2)) == 0)
 	return(if(n1 == 0)
-                   new(class2(cl, "l"), Dim = d, Dimnames = e1@Dimnames)
+		   as(e1, class2(cl, "l"))# more expensive than (but working for "dgC*"):
+		   ## new(class2(cl, "l"), Dim = d, Dimnames = e1@Dimnames)
                else
-		   ## stop(gettextf("<Matrix> %s %s is undefined",
-		   ##     	  .Generic, paste0(class(e2),"(0)")), domain=NA)
 		   as.logical(e2))
     ## else
     if(n1 && n1 < l2)
@@ -166,7 +188,7 @@ Cmp.Mat.atomic <- function(e1, e2) { ## result will inherit from "lMatrix"
 	if(full || allFalse(r0) || extends(cl1, "symmetricMatrix")) {
             isTri <- extends(cl1, "triangularMatrix")
             if(isTri) {
-                if(extends(cl1,"Cholesky") || extends(cl1,"BunchKaufman"))
+                if(extends1of(cl1, c("Cholesky","BunchKaufman")))
                     cl1 <- getClassDef(cl <- class(e1 <- as(e1, "dtrMatrix")))
             }
             ## FIXME? using copyClass() to copy "relevant" slots
@@ -389,23 +411,27 @@ setMethod("Ops", signature(e1="numeric", e2="nMatrix"),
 for(cl in c("numeric", "logical")) { # "complex", "raw" : basically "replValue"
 
 setMethod("Arith", signature(e1 = cl, e2 = "dpoMatrix"),
-	  function(e1, e2) if(length(e1) <= 1 && any(.Generic == c("*","/","+")) && (e1 > 0)) {
-	      e2@x <- callGeneric(e1, e2@x) ; e2 # remains "dpo"
+	  function(e1, e2) if(!(l1 <- length(e1))) numeric()
+			   else if(l1 == 1 && any(.Generic == c("*","/","+")) && (e1 > 0)) {
+	      e2@x <- callGeneric(e1, e2@x) ; .empty.factors(e2); e2 # remains "dpo"
 	  } else ## in all other cases
 	  callGeneric(e1, as(e2, "dsyMatrix")))
 setMethod("Arith", signature(e1 = cl, e2 = "dppMatrix"),
-	  function(e1, e2) if(length(e1) <= 1 && any(.Generic == c("*","/","+")) && (e1 > 0)) {
-	      e2@x <- callGeneric(e1, e2@x) ; e2 # remains "dpp"
+	  function(e1, e2) if(!(l1 <- length(e1))) numeric()
+			   else if(l1 == 1 && any(.Generic == c("*","/","+")) && (e1 > 0)) {
+	      e2@x <- callGeneric(e1, e2@x) ; .empty.factors(e2); e2 # remains "dpp"
 	  } else ## in all other cases
 	  callGeneric(e1, as(e2, "dspMatrix")))
 setMethod("Arith", signature(e1 = "dpoMatrix", e2 = cl),
-	  function(e1, e2) if(length(e2) <= 1 && any(.Generic == c("*","/","+")) && (e2 > 0)) {
-	      e1@x <- callGeneric(e1@x, e2) ; e1 # remains "dpo"
+	  function(e1, e2) if(!(l2 <- length(e2))) numeric()
+			   else if(l2 == 1 && any(.Generic == c("*","/","+")) && (e2 > 0)) {
+	      e1@x <- callGeneric(e1@x, e2) ; .empty.factors(e1); e1 # remains "dpo"
 	  } else ## in all other cases
 	  callGeneric(as(e1, "dsyMatrix"), e2))
 setMethod("Arith", signature(e1 = "dppMatrix", e2 = cl),
-	  function(e1, e2) if(length(e2) <= 1 && any(.Generic == c("*","/","+")) && (e2 > 0)) {
-	      e1@x <- callGeneric(e1@x, e2) ; e1 # remains "dpp"
+	  function(e1, e2) if(!(l2 <- length(e2))) numeric()
+			   else if(l2 == 1 && any(.Generic == c("*","/","+")) && (e2 > 0)) {
+	      e1@x <- callGeneric(e1@x, e2) ; .empty.factors(e1); e1 # remains "dpp"
 	  } else ## in all other cases
 	  callGeneric(as(e1, "dspMatrix"), e2))
 
@@ -436,24 +462,26 @@ setMethod("Arith", signature(e1 = "dgeMatrix", e2 = "dgeMatrix"),
 	      if (!eqD[1])
 		  stop("Matrices must have same number of rows for arithmetic")
 	      same.dim <- eqD[2]
+              x1 <- e1@x
+              x2 <- e2@x
 	      if (same.dim) {
 		  d <- d1
 		  dn <- dimNamesCheck(e1, e2)
 	      }
 	      else { # nrows differ ----> maybe recycling
 		  if(d2[2] %% d1[2] == 0) { # nrow(e2) is a multiple
-		      e1@x <- rep.int(e1@x, d2[2] %/% d1[2])
+		      x1 <- rep.int(x1, d2[2] %/% d1[2])
 		      d <- d2
 		      dn <- e2@Dimnames
 		  } else if(d1[2] %% d2[2] == 0) { # nrow(e1) is a multiple
-		      e2@x <- rep.int(e2@x, d1[2] %/% d2[2])
+		      x2 <- rep.int(x2, d1[2] %/% d2[2])
 		      d <- d1
 		      dn <- e1@Dimnames
 		  } else
 		      stop(gettextf("number of rows are not compatible for %s",
 				    .Generic), domain=NA)
 	      }
-	      new("dgeMatrix", Dim = d, Dimnames = dn, x = callGeneric(e1@x, e2@x))
+	      new("dgeMatrix", Dim = d, Dimnames = dn, x = callGeneric(x1, x2))
 	  })
 
 A.M.n <- function(e1, e2) {
@@ -466,6 +494,7 @@ A.M.n <- function(e1, e2) {
 	    as.numeric(e2)
     else if(le == 1 || le == d[1] || any(prod(d) == c(le, 0L))) { # matching dim
         e1@x <- callGeneric(e1@x, as.vector(e2))
+	.empty.factors(e1)
         e1
     } else stop ("length of 2nd arg does not match dimension of first")
 }
@@ -483,6 +512,7 @@ A.n.M <- function(e1, e2) {
 	    as.numeric(e1)
     else if(le == 1 || le == d[1] || any(prod(d) == c(le, 0L))) { # matching dim
         e2@x <- callGeneric(as.vector(e1), e2@x)
+	.empty.factors(e2)
         e2
     } else stop ("length of 1st arg does not match dimension of 2nd")
 }
@@ -520,6 +550,7 @@ setMethod("Arith", signature(e1 = "ddenseMatrix", e2 = "ddenseMatrix"),
                 if(e1@diag == "U" && !all(1 == callGeneric(1,e2)))
                     e1 <- diagU2N(e1)
                 e1@x <- callGeneric(e1@x, e2)
+		.empty.factors(e1)
                 e1
             } else { ## result *general*
                 callGeneric(as(e1,"dgeMatrix"), e2)
@@ -527,6 +558,7 @@ setMethod("Arith", signature(e1 = "ddenseMatrix", e2 = "ddenseMatrix"),
         } else {                    ## symmetric
             if(le == 1) {           ## result remains symmetric
                 e1@x <- callGeneric(e1@x, e2)
+		.empty.factors(e1)
                 e1
             } else { ## (le == d[1] || prod(d) == le)
                 ## *might* remain symmetric, but 'x' may contain garbage
@@ -556,6 +588,7 @@ setMethod("Arith", signature(e1 = "ddenseMatrix", e2 = "sparseVector"), .Arith.d
 		if(e2@diag == "U" && !all(1 == callGeneric(e1,1)))
 		    e2 <- diagU2N(e2)
 		e2@x <- callGeneric(e1, e2@x)
+		.empty.factors(e2)
 		e2
 	    } else {			# result *general*
 		callGeneric(e1, as(e2,"dgeMatrix"))
@@ -563,6 +596,7 @@ setMethod("Arith", signature(e1 = "ddenseMatrix", e2 = "sparseVector"), .Arith.d
 	} else { ## symmetric
 	    if(le == 1) {		# result remains symmetric
 		e2@x <- callGeneric(e1, e2@x)
+		.empty.factors(e2)
 		e2
 	    } else { ## (le == d[1] || prod(d) == le)
 		## *might* remain symmetric, but 'x' may contain garbage
@@ -671,10 +705,9 @@ Logic.Mat.atomic <- function(e1, e2) { ## result will typically be "like" e1:
     cl <- class(e1)
     if(l2 == 0)
 	return(if(n1 == 0)
-		   new(class2(cl, "l"), Dim = d, Dimnames = e1@Dimnames)
+		   as(e1, class2(cl, "l"))# more expensive than (but working for "dgC*"):
+		   ## new(class2(cl, "l"), Dim = d, Dimnames = e1@Dimnames)
 	       else
-		   ## stop(gettextf("<Matrix> %s %s is undefined",
-		   ##     	  .Generic, paste0(class(e2),"(0)")), domain=NA)
 		   as.logical(e2))
     ## else
     cl1 <- getClassDef(cl)
@@ -703,7 +736,7 @@ Logic.Mat.atomic <- function(e1, e2) { ## result will typically be "like" e1:
 	if(full || allFalse(r0) || extends(cl1, "symmetricMatrix")) {
             isTri <- extends(cl1, "triangularMatrix")
             if(isTri) {
-                if(extends(cl1,"Cholesky") || extends(cl1,"BunchKaufman"))
+                if(extends1of(cl1, c("Cholesky","BunchKaufman")))
                     cl1 <- getClassDef(cl <- class(e1 <- as(e1, "dtrMatrix")))
             }
             ## FIXME? using copyClass() to copy "relevant" slots
@@ -771,7 +804,7 @@ Logic.Mat.atomic <- function(e1, e2) { ## result will typically be "like" e1:
                 nCl <- class2(class(M), 'l') # logical Tsparse
                 sN <- slotNames(nCl)
                 ## copy "the other slots" (important for "tr"/"sym"):
-                r <- copyClass(M, nCl, sNames = sN[is.na(match(sN, "x"))])
+                r <- copyClass(M, nCl, sNames = sN[is.na(match(sN, c("x","factors")))])
 		r@x <- callGeneric(if(has.x) M@x else TRUE, e2)
                 if(extends(cl1, "CsparseMatrix"))
                     r <- as(r, "CsparseMatrix")
@@ -880,6 +913,7 @@ Logic.lCMat <- function(e1, e2, isOR) {
     ## Very easy case first :
     if(identical(e1@i, e2@i) && identical(e1@p, e2@p)) {
         e1@x <- if(isOR) e1@x | e2@x else e1@x & e2@x
+        .empty.factors(e1)
         return(e1)
     }
     ## else :
@@ -899,6 +933,7 @@ Logic.lTMat <- function(e1,e2) {
     ## Very easy case first :
     if(identical(e1@i, e2@i) && identical(e1@j, e2@j)) {
         e1@x <- callGeneric(e1@x, e2@x)
+        .empty.factors(e1)
         return(e1)
     }
     ## else :
@@ -1128,9 +1163,7 @@ setMethod("Arith", signature(e1 = "dtCMatrix", e2 = "dtCMatrix"),
 	       e1@diag == "U" && !all(1 == callGeneric(1, e2)))
 		e1 <- .diagU2N(e1, cld)
 	    e1@x <- callGeneric(e1@x, e2)
-	    if(extends(cld, "compMatrix") && length(e1@factors))
-                ## TODO: be much smarter and try *updating* (some) 'factors':
-		e1@factors <- list()
+	    .empty.factors(e1) # TODO be much smarter and e.g. update U of an LU-factorization
 	    return(e1)
 	}
     }
@@ -1149,9 +1182,7 @@ setMethod("Arith", signature(e1 = "dtCMatrix", e2 = "dtCMatrix"),
 	       e2@diag == "U" && !all(1 == callGeneric(e1, 1)))
 		e2 <- .diagU2N(e2, cld)
 	    e2@x <- callGeneric(e1, e2@x)
-	    if(extends(cld, "compMatrix") && length(e2@factors))
-                ## TODO: be much smarter and try *updating* (some) 'factors':
-		e2@factors <- list()
+	    .empty.factors(e2) # TODO: much smarter, e.g. update U of an LU-factorization
 	    return(e2)
 	}
     }
@@ -1163,7 +1194,7 @@ setMethod("Arith", signature(e1 = "numeric", e2 = "CsparseMatrix"), .Arith.atom.
 ##' compute indices for recycling <numeric> of length 'len' to match sparseMatrix 'spM'
 .Ops.recycle.ind <- function(spM, len) {
     n <- prod(d <- dim(spM))
-    if(n < len) stop("vector too long in Matrix - vector operation")
+    if(n && n < len) stop("vector too long in Matrix - vector operation")
     if(n %% len != 0) ## identical warning as in main/arithmetic.c
         warning("longer object length\n\tis not a multiple of shorter object length")
     ## TODO(speedup!): construction of [1L + in0 %%len] via one .Call()
@@ -1173,25 +1204,22 @@ setMethod("Arith", signature(e1 = "numeric", e2 = "CsparseMatrix"), .Arith.atom.
 }
 
 A.M.n <- function(e1, e2) {
-    if((l2 <- length(e2)) == 0)
-	stop(gettextf("<Matrix> %s %s is undefined",
-		      .Generic, paste0(class(e2),"(0)")), domain=NA)
-    is0f <- is0(f0 <- callGeneric(0, e2))
+    if((l2 <- length(e2)) == 0) # return 0-vector of e1's kind, as matrix()+<0>
+	return(if(length(e1)) vector(.type.kind[.M.kind(e1)]) else e1)
+    is0f <- is0(f0 <- callGeneric(0, e2)) #
     if(all(is0f)) { ## result keeps sparseness structure of e1
 	if(l2 > 1) {  #	 "recycle" e2 "carefully"
 	    e2 <- e2[.Ops.recycle.ind(e1, len = l2)]
 	}
 	e1@x <- callGeneric(e1@x, e2)
-	if(length(e1@factors)) # TODO: be smarter and try *updating* (some) 'factors':
-	    e1@factors <- list()
+        .empty.factors(e1) # TODO: possibly rather *update* LU
 	e1
     } else if(mean(is0f) > 7/8) { ## remain sparse ['7/8' is *somewhat* arbitrary]
 	if(l2 > 1) ## as not all callGeneric(0, e2) is 0, e2 is typically sparse
 	    callGeneric(e1, as(e2, "sparseVector"))
 	else { ## l2 == 1: e2 is "scalar"
 	    e1@x <- callGeneric(e1@x, e2)
-	    if(length(e1@factors)) # TODO: be smarter (see above)
-		e1@factors <- list()
+	    .empty.factors(e1)
 	    e1
 	}
     }
@@ -1213,25 +1241,22 @@ setMethod("Arith", signature(e1 = "dsparseMatrix", e2 = "numeric"), .Arith.CM.at
 setMethod("Arith", signature(e1 = "dsparseMatrix", e2 = "logical"), .Arith.CM.atom)
 
 A.n.M <- function(e1, e2) {
-    if((l1 <- length(e1)) == 0)
-	stop(gettextf("%s %s <Matrix> is undefined",
-		      paste0(class(e2),"(0)"), .Generic), domain=NA)
+    if((l1 <- length(e1)) == 0) # return 0-vector of e2's kind, as <0> + matrix()
+	return(if(length(e2)) vector(.type.kind[.M.kind(e2)]) else e2)
     is0f <- is0(f0 <- callGeneric(e1, 0))
     if(all(is0f)) { ## result keeps sparseness structure of e2
 	if(l1 > 1) {  #	 "recycle" e1 "carefully"
 	    e1 <- e1[.Ops.recycle.ind(e2, len = l1)]
 	}
 	e2@x <- callGeneric(e1, e2@x)
-	if(length(e2@factors))# TODO: be much smarter and try *updating* (some) 'factors':
-	    e2@factors <- list()
+	.empty.factors(e2)
 	e2
     } else if(mean(is0f) > 7/8) { ## remain sparse ['7/8' is *somewhat* arbitrar
 	if(l1 > 1) ## as not all callGeneric(e1, 0) is 0, e1 is typically sparse
 	    callGeneric(as(e1, "sparseVector"), e2)
 	else { ## l1 == 1: e1 is "scalar"
 	    e2@x <- callGeneric(e1, e2@x)
-	    if(length(e2@factors))# TODO: be much smarter (see above)
-		e2@factors <- list()
+	    .empty.factors(e2)
 	    e2
 	}
     }
@@ -1425,7 +1450,12 @@ setMethod("Compare", signature(e1 = "CsparseMatrix", e2 = "CsparseMatrix"),
 
 ## "Arith" short cuts / exceptions
 setMethod("-", signature(e1 = "sparseMatrix", e2 = "missing"),
-	  function(e1, e2) { e1 <- diagU2N(e1); e1@x <- -e1@x; e1 })
+	  function(e1, e2) {
+	      e1 <- diagU2N(e1)
+	      e1@x <- -e1@x
+              .empty.factors(e1)
+	      e1
+	  })
 ## with the following exceptions:
 setMethod("-", signature(e1 = "nsparseMatrix", e2 = "missing"),
 	  function(e1,e2) callGeneric(as(as(as(e1, "CsparseMatrix"), "dMatrix"), "dgCMatrix")))
@@ -1552,13 +1582,16 @@ setMethod("Ops", signature(e1 = "atomicVector", e2 = "sparseVector"),
 Ops.spV.spV <- function(e1, e2) {
     n1 <- e1@length
     n2 <- e2@length
+    if(!n1 || !n2) ## return 0-length :
+	return(if(is.na(match(.Generic, .ArithGenerics))) logical() else numeric())
+    ## else  n1, n2 >= 1 :
     if(n1 != n2) {
 	if(n1 < n2) {
 	    n <- n1 ; N <- n2
 	} else {
 	    n <- n2 ; N <- n1
 	}
-	if(n == 1) {		  # simple case, do not really recycle
+        if(n == 1L) { # simple case, do not really recycle
 	    if(n1 < n2) return(callGeneric(sp2vec(e1), e2))
 	    else	return(callGeneric(e1, sp2vec(e2)))
 	}
@@ -1646,7 +1679,7 @@ Ops.M.spV <- function(e1, e2) {
     n1 <- prod(d)
     n2 <- e2@length
     if(n1 != n2) {
-	if(n1 < n2) {
+	if(n1 && n1 < n2) { # 0-extent matrix + vector is fine
 	    stop(sprintf(
 		"dim [product %d] do not match the length of object [%d]",
 			 n1, n2))
@@ -1672,7 +1705,7 @@ Ops.spV.M <- function(e1, e2) {
     d <- e2@Dim
     n2 <- prod(d)
     if(n2 != n1) {
-	if(n2 < n1) {
+	if(n2 && n2 < n1) { # vector + 0-extent matrix  is fine
 	    stop(sprintf(
 		"dim [product %d] do not match the length of object [%d]",
 			 n2, n1))
