@@ -11,13 +11,14 @@ setAs("dgeMatrix", "matrix", ge2mat)
 ##  "[" settings are "up in"  Matrix.R & denseMatrix.R
 
 
-setMethod("as.vector", signature(x = "dgeMatrix", mode = "missing"),
-          function(x, mode) x@x)
+setMethod("as.vector", "dgeMatrix",
+          function(x, mode) as.vector(x@x, mode))
 
 setMethod("norm", signature(x = "dgeMatrix", type = "missing"),
 	  function(x, type, ...) norm(x, type = "O", ...))
 setMethod("norm", signature(x = "dgeMatrix", type = "character"),
-	  function(x, type, ...) .Call(dgeMatrix_norm, x, type),
+	  function(x, type, ...)
+	      if(identical("2", type)) norm2(x) else .Call(dgeMatrix_norm, x, type),
 	  valueClass = "numeric")
 
 setMethod("rcond", signature(x = "dgeMatrix", norm = "missing"),
@@ -35,14 +36,31 @@ setMethod("rcond", signature(x = "dgeMatrix", norm = "character"),
 ##> Hence, keep the Matrix version active for now:
 ##> if(getRversion() < "2.11.0" || R.version$`svn rev` < 51018)
 ##--- the same for "traditional"  'matrix':
-setMethod("norm", signature(x = "matrix", type = "character"),
-	  function(x, type, ...) .Call(dgeMatrix_norm, ..2dge(x), type),
-	  valueClass = "numeric")
+## 2017-02-08: Rather keep using base norm for 'matrix'
+## setMethod("norm", signature(x = "matrix", type = "character"),
+## 	  function(x, type, ...) .Call(dgeMatrix_norm, ..2dge(x), type),
+## 	  valueClass = "numeric")
 
 setMethod("t", signature(x = "dgeMatrix"), t_geMatrix)
 
+..get.diag <- function(x, nrow, ncol, names=TRUE) {
+    ##         vvvvvvvvvvvvvvvvv here just a place holder, replaced in .mkSpec.diag()
+    y <- .Call(dgeMatrix_getDiag, x) # double or logical
+    if(names) {
+        nms <- dimnames(x)
+        if(is.list(nms) && !any(vapply(nms, is.null, NA)) &&
+           identical((nm <- nms[[1L]][im <- seq_len(min(dim(x)))]), nms[[2L]][im]))
+            names(y) <- nm
+    }
+    y
+}
+.mkSpec.diag <- function(symb) {
+    rr <- ..get.diag
+    body(rr)[[2]][[3]][[2]] <- symb
+    rr
+}
+.dge.diag <- .mkSpec.diag(quote(dgeMatrix_getDiag))
 
-.dge.diag <- function(x, nrow, ncol) .Call(dgeMatrix_getDiag, x)
 setMethod("diag", signature(x = "dgeMatrix"), .dge.diag)
 setMethod("diag<-", signature(x = "dgeMatrix"),
 	  function(x, value) .Call(dgeMatrix_setDiag, x, value))
