@@ -42,7 +42,7 @@ setMethod("determinant", signature(x = "dtrMatrix", logarithm = "logical"),
 	  function(x, logarithm, ...) mkDet(diag(x), logarithm))
 
 setMethod("diag", signature(x = "dtrMatrix"),
-          function(x, nrow, ncol) .Call(dtrMatrix_getDiag, x),
+          .mkSpec.diag(quote(dtrMatrix_getDiag)),
           valueClass = "numeric")
 setMethod("diag<-", signature(x = "dtrMatrix"),
 	  function(x, value) {
@@ -53,7 +53,7 @@ setMethod("diag<-", signature(x = "dtrMatrix"),
 
 setMethod("norm", signature(x = "dtrMatrix", type = "character"),
 	  function(x, type, ...)
-	  .Call(dtrMatrix_norm, x, type),
+	      if(identical("2", type)) norm2(x) else .Call(dtrMatrix_norm, x, type),
 	  valueClass = "numeric")
 
 setMethod("norm", signature(x = "dtrMatrix", type = "missing"),
