@@ -771,6 +771,8 @@ setMethod("isSymmetric", signature(object = "sparseMatrix"),
 setMethod("isTriangular", signature(object = "CsparseMatrix"), isTriC)
 setMethod("isTriangular", signature(object = "TsparseMatrix"), isTriT)
 
+## no longer used for "Csparse*" which has own method in ./Csparse.R , nor
+##                for "Tsparse*" which has own method in ./Tsparse.R ... so only for Rsparse*?
 setMethod("isDiagonal", signature(object = "sparseMatrix"),
 	  function(object) {
               d <- dim(object)
@@ -794,7 +796,7 @@ setMethod("Cholesky", signature(A = "sparseMatrix"),
 		   perm=perm, LDL=LDL, super=super, Imult=Imult, ...))
 
 setMethod("diag", signature(x = "sparseMatrix"),
-	  function(x, nrow, ncol) diag(as(x, "CsparseMatrix")))
+	  function(x, nrow, ncol, names=TRUE) diag(as(x, "CsparseMatrix"), names=names))
 
 setMethod("dim<-", signature(x = "sparseMatrix", value = "ANY"),
 	  function(x, value) {
