@@ -23,47 +23,46 @@
     }
 }
 
+Rv <- getRversion() # removed at end
+
 ## Instead, simply re-assign the [cr]bind()s which are recursively
 ## based on [cr]bind2 :
 ##
 ## save to cBind / rBind  ("rename")
-if(getRversion() >= "3.2.0") {
-    ## New (2015-02)  base :: cbind(), rbind() which dispatch on S4 "when needed":
+## R >= "3.2.0" :
+## New (2015-02)  base :: cbind(), rbind() which dispatch on S4 "when needed":
     cBind <- function (..., deparse.level = 1) {
-	## Once per session warning (or if "Matrix.(warn|verbose)"):
-	if(is.null(wrn <- get0("warned.cBind", .MatrixEnv)) ||
-	   isTRUE(getOption("Matrix.warn")) ||
-	   isTRUE(getOption("Matrix.verbose"))) {
-	    if(is.null(wrn))
-		assign("warned.cBind", TRUE, envir=.MatrixEnv)
-	    .Deprecated(msg = "'cBind' is deprecated.
+	    .Defunct(msg = "'cBind' is defunct.
  Since R version 3.2.0, base's cbind() should work fine with S4 objects")
-	}
-	base::cbind(..., deparse.level=deparse.level)
     }
     rBind <- function (..., deparse.level = 1) {
-	## Once per session warning (or if "Matrix.(warn|verbose)"):
-	if(is.null(wrn <- get0("warned.rBind", .MatrixEnv)) ||
-           isTRUE(getOption("Matrix.warn")) ||
-	   isTRUE(getOption("Matrix.verbose"))) {
-	    if(is.null(wrn))
-		assign("warned.rBind", TRUE, envir=.MatrixEnv)
-	    .Deprecated(msg = "'rBind' is deprecated.
+	    .Defunct(msg = "'rBind' is defunct.
  Since R version 3.2.0, base's rbind() should work fine with S4 objects")
-	}
-	base::rbind(..., deparse.level=deparse.level)
     }
 
-} else {
-    cBind <- methods:::cbind
-    rBind <- methods:::rbind
-    lengths <- function (x, use.names = TRUE) vapply(x, length, 1L, USE.NAMES = use.names)
-}
+if(Rv < "4.0.0") {
+  deparse1 <- function (expr, collapse = " ", width.cutoff = 500L, ...)
+      paste(deparse(expr, width.cutoff, ...), collapse = collapse)
+  ## not equivalent ...
+  ...length <- function() eval(quote(length(list(...))), sys.frame(-1L))
 
-if(getRversion() < "3.5.0") {
+  tryInvokeRestart <- function(r, ...) {
+    if (!isRestart(r))
+        r <- findRestart(r)
+
+    if (is.null(r))
+        invisible(NULL)
+    else
+        .Internal(.invokeRestart(r, list(...)))
+  }
+
+  if(Rv < "3.5.0") {
     isFALSE <- function (x) is.logical(x) && length(x) == 1L && !is.na(x) && !x
     isTRUE  <- function (x) is.logical(x) && length(x) == 1L && !is.na(x) && x
+  }
 }
+rm(Rv)
+
 
 .onUnload <- function(libpath)
 {
@@ -76,10 +75,3 @@ if(getRversion() < "3.5.0") {
     ssv <- .Call(get_SuiteSparse_version)
     package_version(list(major = ssv[1], minor = paste(ssv[2:3], collapse=".")))
 }
-
-if(getRversion() < "3.1.0") {
-    if(getRversion() < "3.0.0") {
-        rep_len <- function(x, length.out) rep(x, length.out=length.out)
-    }
-    anyNA <- function(x) any(is.na(x))
-}
