@@ -60,21 +60,26 @@ static CSP csp_eye(int n)
  */
 cs *Matrix_as_cs(cs *ans, SEXP x, Rboolean check_Udiag)
 {
-    static const char *valid[] = {"dgCMatrix", "dtCMatrix", ""};
+    static const char *valid[] =
+	{ "dgCMatrix", "dtCMatrix"
+	  // 0            1
+	, "ngCMatrix", "ntCMatrix"
+	  // 2            3
+	, ""};
     /* had also "dsCMatrix", but that only stores one triangle */
-    int *dims, ctype = Matrix_check_class_etc(x, valid);
-    SEXP islot;
-
+    int ctype = R_check_class_etc(x, valid);
     if (ctype < 0) error(_("invalid class of 'x' in Matrix_as_cs(a, x)"));
+    // now, ctype >= 0
+    Rboolean has_x = ctype < 2;
 				/* dimensions and nzmax */
-    dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
+    int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
     ans->m = dims[0]; ans->n = dims[1];
-    islot = GET_SLOT(x, Matrix_iSym);
+    SEXP islot = GET_SLOT(x, Matrix_iSym);
     ans->nz = -1;		/* indicates compressed column storage */
     ans->nzmax = LENGTH(islot);
     ans->i = INTEGER(islot);
     ans->p = INTEGER(GET_SLOT(x, Matrix_pSym));
-    ans->x = REAL(GET_SLOT(x, Matrix_xSym));
+    ans->x = has_x ? REAL(GET_SLOT(x, Matrix_xSym)) : NULL;
 
     if(check_Udiag && ctype == 1 && (*diag_P(x) == 'U')) { /* diagU2N(.) : */
 	int n = dims[0];
@@ -94,11 +99,12 @@ cs *Matrix_as_cs(cs *ans, SEXP x, Rboolean check_Udiag)
 	/* content(ans) := content(tmp) : */
 	ans->nzmax = nz;
 	/* The ans "slots" were pointers to x@ <slots>; all need new content now: */
-	ans->p = Memcpy((   int*) R_alloc(sizeof(   int), n+1),
+	ans->p = Memcpy((   int*) R_alloc(n+1, sizeof(int)),
 			(   int*) tmp->p, n+1);
-	ans->i = Memcpy((   int*) R_alloc(sizeof(   int), nz),
+	ans->i = Memcpy((   int*) R_alloc(nz, sizeof(int)),
 			(   int*) tmp->i, nz);
-	ans->x = Memcpy((double*) R_alloc(sizeof(double), nz),
+	if(has_x)
+	ans->x = Memcpy((double*) R_alloc(nz, sizeof(double)),
 			(double*) tmp->x, nz);
 
 	cs_spfree(tmp);
@@ -112,21 +118,23 @@ cs *Matrix_as_cs(cs *ans, SEXP x, Rboolean check_Udiag)
  *
  * @param a matrix to be converted
  * @param cl the name of the S4 class of the object to be generated
- * @param dofree 0 - don't free a; > 0 cs_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cs_free a; < 0 R_Free a
+ * @param dn either R_NilValue or an SEXP suitable for the Dimnames slot.
  *
  * @return SEXP containing a copy of a
  */
-/* FIXME:  Change API : May need object,  not just class name 'cl' */
-SEXP Matrix_cs_to_SEXP(cs *a, char *cl, int dofree)
+// FIXME: Change API : Use object,  not just class name 'cl' -- and use R_check_class(obj, *)
+SEXP Matrix_cs_to_SEXP(cs *a, char *cl, int dofree, SEXP dn)
 {
     static const char *valid[] = {"dgCMatrix", "dsCMatrix", "dtCMatrix", ""};
     int ctype = Matrix_check_class(cl, valid);
 
     if (ctype < 0)
 	error(_("invalid class of object to %s"), "Matrix_cs_to_SEXP");
-    SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cl)));
+    SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS(cl));
 				/* allocate and copy common slots */
     int *dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
+    PROTECT(dn); // <- as in chm_sparse_to_SEXP()
     dims[0] = a->m; dims[1] = a->n;
     Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_pSym, INTSXP, a->n + 1)),
 	   a->p, a->n + 1);
@@ -142,8 +150,10 @@ SEXP Matrix_cs_to_SEXP(cs *a, char *cl, int dofree)
 	SET_SLOT(ans, Matrix_uploSym, mkString(uplo < 0 ? "L" : "U"));
     }
     if (dofree > 0) cs_spfree(a);
-    if (dofree < 0) Free(a);
-    UNPROTECT(1);
+    if (dofree < 0) R_Free(a);
+    if (dn != R_NilValue)
+	SET_SLOT(ans, Matrix_DimNamesSym, duplicate(dn));
+    UNPROTECT(2);
     return ans;
 }
 
@@ -203,15 +213,15 @@ csn *Matrix_as_csn(csn *ans, SEXP x)
 
     if (ctype < 0)
 	error(_("invalid class of object to %s"), "Matrix_as_csn");
-    ans->U = Matrix_as_cs(GET_SLOT(x, install("U")));
-    ans->L = Matrix_as_cs(GET_SLOT(x, install("L")));
+    ans->U = Matrix_as_cs(GET_SLOT(x, Matrix_USym));
+    ans->L = Matrix_as_cs(GET_SLOT(x, Matrix_LSym));
     switch(ctype) {
     case 0:
 	ans->B = (double*) NULL;
 	ans->pinv = INTEGER(GET_SLOT(x, install("Pinv")));
 	break;
     case 1:
-	ans->B = REAL(GET_SLOT(x, install("beta")));
+	ans->B = REAL(GET_SLOT(x, Matrix_betaSym));
 	ans->pinv = (int*) NULL;
 	break;
     default:
@@ -226,7 +236,7 @@ csn *Matrix_as_csn(csn *ans, SEXP x)
  *
  * @param a css object to be converted
  * @param cl the name of the S4 class of the object to be generated
- * @param dofree 0 - don't free a; > 0 cs_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cs_free a; < 0 R_Free a
  * @param m number of rows in original matrix
  * @param n number of columns in original matrix
  *
@@ -241,7 +251,7 @@ SEXP Matrix_css_to_SEXP(css *S, char *cl, int dofree, int m, int n)
     if (ctype < 0)
 	error(_("Inappropriate class cl='%s' in Matrix_css_to_SEXP(S, cl, ..)"),
 	      cl);
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cl)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(cl));
 				/* allocate and copy common slots */
     Memcpy(INTEGER(ALLOC_SLOT(ans, install("Q"), INTSXP, n)), S->q, n);
     nz = INTEGER(ALLOC_SLOT(ans, install("nz"), INTSXP, 3));
@@ -262,7 +272,7 @@ SEXP Matrix_css_to_SEXP(css *S, char *cl, int dofree, int m, int n)
 	      cl);
     }
     if (dofree > 0) cs_sfree(S);
-    if (dofree < 0) Free(S);
+    if (dofree < 0) R_Free(S);
     UNPROTECT(1);
     return ans;
 }
@@ -273,11 +283,12 @@ SEXP Matrix_css_to_SEXP(css *S, char *cl, int dofree, int m, int n)
  *
  * @param a csn object to be converted
  * @param cl the name of the S4 class of the object to be generated
- * @param dofree 0 - don't free a; > 0 cs_free a; < 0 Free a
+ * @param dofree 0 - don't free a; > 0 cs_free a; < 0 R_Free a
+ * @param dn either R_NilValue or an SEXP suitable for the Dimnames slot. FIXME (L,U!)
  *
  * @return SEXP containing a copy of S
  */
-SEXP Matrix_csn_to_SEXP(csn *N, char *cl, int dofree)
+SEXP Matrix_csn_to_SEXP(csn *N, char *cl, int dofree, SEXP dn)
 {
     SEXP ans;
     static const char *valid[] = {"csn_LU", "csn_QR", ""};
@@ -286,20 +297,20 @@ SEXP Matrix_csn_to_SEXP(csn *N, char *cl, int dofree)
     if (ctype < 0)
 	error(_("Inappropriate class cl='%s' in Matrix_csn_to_SEXP(S, cl, ..)"),
 	      cl);
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cl)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(cl));
 				/* allocate and copy common slots */
     /* FIXME: Use the triangular matrix classes for csn_LU */
-    SET_SLOT(ans, install("L"),	/* these are free'd later if requested */
-	     Matrix_cs_to_SEXP(N->L, "dgCMatrix", 0));
-    SET_SLOT(ans, install("U"),
-	     Matrix_cs_to_SEXP(N->U, "dgCMatrix", 0));
+    SET_SLOT(ans, Matrix_LSym,	/* these are free'd later if requested */
+	     Matrix_cs_to_SEXP(N->L, "dgCMatrix", 0, dn)); // FIXME: dn
+    SET_SLOT(ans, Matrix_USym,
+	     Matrix_cs_to_SEXP(N->U, "dgCMatrix", 0, dn)); // FIXME: dn
     switch(ctype) {
     case 0:
 	Memcpy(INTEGER(ALLOC_SLOT(ans, install("Pinv"), INTSXP, n)),
 	       N->pinv, n);
 	break;
     case 1:
-	Memcpy(REAL(ALLOC_SLOT(ans, install("beta"), REALSXP, n)),
+	Memcpy(REAL(ALLOC_SLOT(ans, Matrix_betaSym, REALSXP, n)),
 	       N->B, n);
 	break;
     default:
@@ -308,7 +319,7 @@ SEXP Matrix_csn_to_SEXP(csn *N, char *cl, int dofree)
     }
     if (dofree > 0) cs_nfree(N);
     if (dofree < 0) {
-	Free(N->L); Free(N->U); Free(N);
+	R_Free(N->L); R_Free(N->U); R_Free(N);
     }
     UNPROTECT(1);
     return ans;
