@@ -6,8 +6,8 @@ SEXP dpoMatrix_validate(SEXP obj)
     if (isString(val = dense_nonpacked_validate(obj)))
 	return(val);
 
-    int n = INTEGER(GET_SLOT(obj, Matrix_DimSym))[0],
-	np1 = n + 1;
+    int n = INTEGER(GET_SLOT(obj, Matrix_DimSym))[0];
+    R_xlen_t np1 = n + 1;
     double *x = REAL(GET_SLOT(obj, Matrix_xSym));
 
     /* quick but nondefinitive check on positive definiteness */
@@ -25,6 +25,7 @@ SEXP dpoMatrix_chol(SEXP x)
     const char *uplo = CHAR(STRING_ELT(uploP, 0));
     int *dims = INTEGER(dimP), info;
     int n = dims[0];
+    const R_xlen_t n2 = ((R_xlen_t)n) * n; // = n^2
     double *vx;
 
     if (val != R_NilValue) return val;// use x@factors$Cholesky if available
@@ -33,11 +34,12 @@ SEXP dpoMatrix_chol(SEXP x)
     SET_SLOT(val, Matrix_uploSym, duplicate(uploP));
     SET_SLOT(val, Matrix_diagSym, mkString("N"));
     SET_SLOT(val, Matrix_DimSym, duplicate(dimP));
-    vx = REAL(ALLOC_SLOT(val, Matrix_xSym, REALSXP, n * n));
-    AZERO(vx, n * n);
-    F77_CALL(dlacpy)(uplo, &n, &n, REAL(GET_SLOT(x, Matrix_xSym)), &n, vx, &n);
+    vx = REAL(ALLOC_SLOT(val, Matrix_xSym, REALSXP, n2));
+    AZERO(vx, n2);
+    F77_CALL(dlacpy)(uplo, &n, &n, REAL(GET_SLOT(x, Matrix_xSym)),
+		     &n, vx, &n FCONE);
     if (n > 0) {
-	F77_CALL(dpotrf)(uplo, &n, vx, &n, &info);
+	F77_CALL(dpotrf)(uplo, &n, vx, &n, &info  FCONE);
 	if (info) {
 	    if(info > 0)
 		error(_("the leading minor of order %d is not positive definite"),
@@ -61,7 +63,7 @@ SEXP dpoMatrix_rcond(SEXP obj, SEXP type)
 		     dims, REAL(GET_SLOT(Chol, Matrix_xSym)),
 		     dims, &anorm, &rcond,
 		     (double *) R_alloc(3*dims[0], sizeof(double)),
-		     (int *) R_alloc(dims[0], sizeof(int)), &info);
+		     (int *) R_alloc(dims[0], sizeof(int)), &info FCONE);
     return ScalarReal(rcond);
 }
 
@@ -78,7 +80,7 @@ SEXP dpoMatrix_solve(SEXP x)
     SET_SLOT(val, Matrix_DimNamesSym,
 	     duplicate(GET_SLOT(x, Matrix_DimNamesSym)));
     F77_CALL(dpotri)(uplo_P(val), dims,
-		     REAL(GET_SLOT(val, Matrix_xSym)), dims, &info);
+		     REAL(GET_SLOT(val, Matrix_xSym)), dims, &info FCONE);
     UNPROTECT(1);
     return val;
 }
@@ -101,7 +103,7 @@ SEXP dpoMatrix_dgeMatrix_solve(SEXP a, SEXP b)
     F77_CALL(dpotrs)(uplo_P(Chol), adims, bdims + 1,
 		     REAL(GET_SLOT(Chol, Matrix_xSym)), adims,
 		     REAL(GET_SLOT(val, Matrix_xSym)),
-		     bdims, &info);
+		     bdims, &info FCONE);
     UNPROTECT(1);
     return val;
 }
@@ -120,7 +122,7 @@ SEXP dpoMatrix_matrix_solve(SEXP a, SEXP b)
 	error(_("Dimensions of system to be solved are inconsistent"));
     F77_CALL(dpotrs)(uplo_P(Chol), adims, bdims + 1,
 		     REAL(GET_SLOT(Chol, Matrix_xSym)), adims,
-		     REAL(val), bdims, &info);
+		     REAL(val), bdims, &info FCONE);
     UNPROTECT(1);
     return val;
 }
