@@ -26,7 +26,7 @@ double get_norm_sp(SEXP obj, const char *typstr)
 	work = (double *) R_alloc(dims[0], sizeof(double));
     }
     return F77_CALL(dlansp)(typnm, uplo_P(obj), dims,
-			    REAL(GET_SLOT(obj, Matrix_xSym)), work);
+			    REAL(GET_SLOT(obj, Matrix_xSym)), work FCONE FCONE);
 }
 
 SEXP dspMatrix_norm(SEXP obj, SEXP type)
@@ -45,7 +45,7 @@ SEXP dspMatrix_rcond(SEXP obj, SEXP type)
 		     INTEGER(GET_SLOT(trf, Matrix_permSym)),
 		     &anorm, &rcond,
 		     (double *) R_alloc(2*dims[0], sizeof(double)),
-		     (int *) R_alloc(dims[0], sizeof(int)), &info);
+		     (int *) R_alloc(dims[0], sizeof(int)), &info FCONE);
     return ScalarReal(rcond);
 }
 
@@ -61,7 +61,7 @@ SEXP dspMatrix_solve(SEXP a)
     F77_CALL(dsptri)(uplo_P(val), dims, REAL(GET_SLOT(val, Matrix_xSym)),
 		     INTEGER(GET_SLOT(trf, Matrix_permSym)),
 		     (double *) R_alloc((long) dims[0], sizeof(double)),
-		     &info);
+		     &info FCONE);
     UNPROTECT(1);
     return val;
 }
@@ -79,11 +79,15 @@ SEXP dspMatrix_matrix_solve(SEXP a, SEXP b)
     F77_CALL(dsptrs)(uplo_P(trf),
 		     &n, &nrhs, REAL(GET_SLOT(trf, Matrix_xSym)),
 		     INTEGER(GET_SLOT(trf, Matrix_permSym)),
-		     REAL(GET_SLOT(val, Matrix_xSym)), &n, &info);
+		     REAL(GET_SLOT(val, Matrix_xSym)), &n, &info FCONE);
     UNPROTECT(1);
     return val;
 }
 
+
+/* MJ: No longer needed ... replacement in ./packedMatrix.c */
+#if 0
+
 SEXP dspMatrix_getDiag(SEXP x)
 
 {
@@ -117,6 +121,8 @@ SEXP lspMatrix_setDiag(SEXP x, SEXP d)
     return l_packed_setDiag(INTEGER(d), LENGTH(d), x, n);
 }
 
+#endif /* MJ */
+
 
 SEXP dspMatrix_as_dsyMatrix(SEXP from)
 {
@@ -141,22 +147,22 @@ SEXP dspMatrix_matrix_mm(SEXP a, SEXP b)
     SEXP val = PROTECT(dup_mMatrix_as_dgeMatrix(b));
     int *bdims = INTEGER(GET_SLOT(val, Matrix_DimSym));
     int i, ione = 1, n = bdims[0], nrhs = bdims[1];
-    double nn = ((double) n) * ((double) nrhs);
-    if (nn > INT_MAX)
-	error(_("Matrix dimension %d x %d (= %g) is too large"), n, nrhs, nn);
+    R_xlen_t nn = n * (R_xlen_t) nrhs;
     const char *uplo = uplo_P(a);
     double *ax = REAL(GET_SLOT(a, Matrix_xSym)), one = 1., zero = 0.,
 	*vx = REAL(GET_SLOT(val, Matrix_xSym)), *bx;
-    C_or_Alloca_TO(bx, n * nrhs, double);
 
-    Memcpy(bx, vx, n * nrhs);
+    C_or_Alloca_TO(bx, nn, double);
+    Memcpy(bx, vx, nn);
     if (bdims[0] != n)
 	error(_("Matrices are not conformable for multiplication"));
     if (nrhs >= 1 && n >= 1) {
-	for (i = 0; i < nrhs; i++)
-	    F77_CALL(dspmv)(uplo, &n, &one, ax, bx + i * n, &ione,
-			    &zero, vx + i * n, &ione);
-	if(n * nrhs >= SMALL_4_Alloca) Free(bx);
+	R_xlen_t in;
+	for (i = 0, in = 0; i < nrhs; i++, in += n) { // in := i * n (w/o overflow!)
+	    F77_CALL(dspmv)(uplo, &n, &one, ax, bx + in, &ione,
+			    &zero, vx + in, &ione FCONE);
+	}
+	if(nn >= SMALL_4_Alloca) R_Free(bx);
     }
     UNPROTECT(1);
     return val;
@@ -179,7 +185,7 @@ SEXP dspMatrix_trf(SEXP x)
     SET_SLOT(val, Matrix_DimSym, duplicate(dimP));
     slot_dup(val, x, Matrix_xSym);
     perm = INTEGER(ALLOC_SLOT(val, Matrix_permSym, INTSXP, n));
-    F77_CALL(dsptrf)(uplo, dims, REAL(GET_SLOT(val, Matrix_xSym)), perm, &info);
+    F77_CALL(dsptrf)(uplo, dims, REAL(GET_SLOT(val, Matrix_xSym)), perm, &info FCONE);
     if (info) error(_("Lapack routine %s returned error code %d"), "dsptrf", info);
     UNPROTECT(1);
     return set_factors(x, val, "pBunchKaufman");
