@@ -1,4 +1,5 @@
-#include "Mutils.h"
+#include <Rinternals.h>
+#include <R_ext/Lapack.h>
 #include "abIndex.h"
 #include "chm_common.h"
 #include "CHMfactor.h"
@@ -22,6 +23,7 @@
 #include "ldense.h"
 #include "lgCMatrix.h"
 #include "sparseQR.h"
+#include "packedMatrix.h"
 #include <R_ext/Rdynload.h>
 
 #include "Syms.h"
@@ -73,11 +75,13 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(Csparse_validate, 1),
     CALLDEF(Csparse_validate2, 2),
     CALLDEF(Csparse_vertcat, 2),
+    CALLDEF(Csparse_dmperm, 3),
     CALLDEF(pCholesky_validate, 1),
     CALLDEF(Rsparse_validate, 1),
     CALLDEF(diag_tC, 2),
     CALLDEF(LU_expand, 1),
     CALLDEF(LU_validate, 1),
+    CALLDEF(matrix_to_Csparse, 2),
     CALLDEF(MatrixFactorization_validate, 1),
     CALLDEF(Matrix_expand_pointers, 1),
     CALLDEF(R_rbind2_vector, 2),
@@ -186,11 +190,16 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(dspMatrix_solve, 1),
     CALLDEF(dspMatrix_trf, 1),
     CALLDEF(dspMatrix_validate, 1),
+
+/* MJ: No longer needed ... replacement in ./packedMatrix.c */
+#if 0
     CALLDEF(dspMatrix_getDiag, 1),
     CALLDEF(lspMatrix_getDiag, 1),
     CALLDEF(dspMatrix_setDiag, 2),
     CALLDEF(lspMatrix_setDiag, 2),
-/*     CALLDEF(dtCMatrix_solve, 1), */
+#endif
+    
+    /* CALLDEF(dtCMatrix_solve, 1), */
     CALLDEF(dtCMatrix_matrix_solve, 3),
     CALLDEF(dtCMatrix_sparse_solve, 2),
     CALLDEF(dtTMatrix_as_dtrMatrix, 1),
@@ -198,10 +207,15 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(ntTMatrix_as_ntrMatrix, 1),
     CALLDEF(dtpMatrix_as_dtrMatrix, 1),
     CALLDEF(dtpMatrix_addDiag, 2),
+
+/* MJ: No longer needed ... replacement in ./packedMatrix.c */
+#if 0
     CALLDEF(dtpMatrix_getDiag, 1),
     CALLDEF(ltpMatrix_getDiag, 1),
     CALLDEF(dtpMatrix_setDiag, 2),
     CALLDEF(ltpMatrix_setDiag, 2),
+#endif
+    
     CALLDEF(dtpMatrix_matrix_mm, 4),
     CALLDEF(dtpMatrix_matrix_solve, 2),
     CALLDEF(dtpMatrix_norm, 2),
@@ -253,8 +267,16 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(sparseQR_resid_fitted, 3),
     CALLDEF(triangularMatrix_validate, 1),
     CALLDEF(symmetricMatrix_validate, 1),
+    CALLDEF(packedMatrix_validate, 1),
     CALLDEF(R_symmetric_Dimnames, 1),
 
+    CALLDEF(packedMatrix_t, 1),
+    CALLDEF(packedMatrix_diag_get, 2),
+    CALLDEF(packedMatrix_diag_set, 2),
+    CALLDEF(packedMatrix_sub1, 2),
+    CALLDEF(packedMatrix_sub1_mat, 2),
+    CALLDEF(packedMatrix_sub2, 4),
+
 /* still simple placeholders, but already used in ../R/AllClass.R : */
     CALLDEF(CHMfactor_validate, 1),
     CALLDEF(CHMsimpl_validate, 1),
@@ -266,10 +288,12 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(m_encodeInd,  4),
     CALLDEF(m_encodeInd2, 5),
 
+    CALLDEF(matrix_trf, 2),
     CALLDEF(Matrix_rle_i, 2),
     CALLDEF(Matrix_rle_d, 2),
 
     CALLDEF(R_set_factors, 4),
+    CALLDEF(R_empty_factors, 2),
 
     CALLDEF(get_SuiteSparse_version, 0),
     {NULL, NULL, 0}
@@ -364,6 +388,9 @@ R_init_Matrix(DllInfo *dll)
     Matrix_permSym = install("perm");
     Matrix_uploSym = install("uplo");
     Matrix_xSym = install("x");
+    Matrix_LSym = install("L");
+    Matrix_RSym = install("R");
+    Matrix_USym = install("U");
     Matrix_VSym = install("V");
 
     Matrix_NS = R_FindNamespace(mkString("Matrix"));
