@@ -919,4 +919,15 @@ stopifnot(identical(dim(m02 %*% Diagonal(x=c(1,2))), c(0L, 2L)),
           identical(dim(M02 %*% Diagonal(2)       ), c(0L, 2L)),
           identical(dim(M02 %*% Diagonal(x=2:1)   ), c(0L, 2L)))
 
+## RsparseMatrix --- Arko Bose (Jan.2022): "Method for <dgRMatrix> %*% <dgCMatrix>"
+m <- Matrix(c(0,0,2:0), 3,5)
+(R <- as(m, "RsparseMatrix"))
+stopifnot(exprs = {
+    all.equal(t(R) %*% R,  crossprod(R))
+    all.equal(R %*% t(R), tcrossprod(R)) # both dgC {latter could improve to dsC*}
+    all.equal(as(R %*% t(m),"symmetricMatrix"), tcrossprod(m))
+    all.equal(as(m %*% t(R),"symmetricMatrix"), tcrossprod(m))
+})
+
+
 cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''
